==> 1.0.0.tar.gz: ios/Classes/sdk/WluperSDK.swift <==
//
//  WluperSDK.swift
//  test_wluper_sdk
//
//  Created by Alfonso Farruggia on 17/09/2021.
//

import Foundation
import AVFoundation
import Speech
import SwiftUI


@available(iOS 10, *)
class WluperSDK {
    
    var listener:WluperInterface;
    var x_api_key:String;
    var transcription:String=""
    var session_id:String=""
    var api:TUApi
    var wluperLogManager: WluperLogManager?
    private let assistant = SpeechAssist()
    private var timer:Timer?
    
    init(_listener : WluperInterface, _x_api_key:String){
        listener=_listener
        x_api_key=_x_api_key
        api = TUApi(xapikey: _x_api_key)
    }

    func setWluperLogBase(_wluperLog: WluperLog){
        wluperLogManager = WluperLogManager(wluperLog:_wluperLog);
    }
    
    @available(iOS 10, *)
    private class SpeechAssist :NSObject , SFSpeechRecognitionTaskDelegate {
        var audioEngine: AVAudioEngine?
        var recognitionRequest: SFSpeechAudioBufferRecognitionRequest?
        var recognitionTask: SFSpeechRecognitionTask?
        let speechRecognizer = SFSpeechRecognizer(locale: Locale(identifier: "en-US"))

        deinit {
            reset()
        }

        func reset() {
            recognitionTask?.cancel()
            audioEngine?.stop()
            audioEngine = nil
            recognitionRequest = nil
            recognitionTask = nil
        }
    }

    func startListening() {
        self.session_id = UUID().uuidString
        var _error = false;
        
        canAccess { [self] authorized in
            guard authorized else {
                self.listener.WluperError(error: "Access denied")
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Access denied", session_id:self.session_id)
                _error = true;
                return
            }

            self.assistant.audioEngine = AVAudioEngine()
            
            guard let audioEngine = self.assistant.audioEngine else {
                self.listener.WluperError(error: "Unable to create audio engine")
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Unable to create audio engine", session_id:self.session_id)
                _error = true
                fatalError("Unable to create audio engine")
            }
            self.assistant.recognitionRequest = SFSpeechAudioBufferRecognitionRequest()
            guard let recognitionRequest = self.assistant.recognitionRequest else {
                self.listener.WluperError(error: "Unable to create request")
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Unable to create request", session_id:self.session_id)
                _error = true
                fatalError("Unable to create request")
            }
            recognitionRequest.shouldReportPartialResults = true
            if #available(iOS 13, *) {
                recognitionRequest.requiresOnDeviceRecognition = true
            }
            
            do {
                let audioSession = AVAudioSession.sharedInstance()
                try audioSession.setCategory(.record, mode: .measurement, options: .duckOthers)
                try audioSession.setActive(true, options: .notifyOthersOnDeactivation)
                let inputNode = audioEngine.inputNode

                let recordingFormat = inputNode.outputFormat(forBus: 0)
                inputNode.installTap(onBus: 0, bufferSize: 1024, format: recordingFormat) { (buffer: AVAudioPCMBuffer, when: AVAudioTime) in
                    recognitionRequest.append(buffer)
                }
                
                audioEngine.prepare()
                try audioEngine.start()

                assistant.recognitionTask = assistant.speechRecognizer?.recognitionTask(with: recognitionRequest ) { (result, error) in
                    if let result = result {
                        self.transcription = result.bestTranscription.formattedString
                        self.listener.WluperLiveText(text: self.transcription)
                        if (self.timer != nil){
                            self.timer?.invalidate();
                        }
                        self.timer = Timer.scheduledTimer(timeInterval: 2.0, target: self, selector: #selector(self.stopListening), userInfo: nil, repeats: false)
                    }
                    
                }
            } catch {
                self.listener.WluperError(error: "Error transcibing audio: " + error.localizedDescription)
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Error transcibing audio: " + error.localizedDescription, session_id:self.session_id)
                _error = true
                assistant.reset()
            }
        }
        if (_error==false){
            self.listener.WluperASR(plain_text:"SpeechRecognizer: onReadyForSpeech");
            self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_KEY.rawValue,event_value:WluperLogKey.ASR_START_LISTENING.rawValue, session_id:self.session_id)

        }

    }

    func wluperTU(text:String){
        let tree:[String:Any] = self.api.understand(text: text)
        
        if let error = tree["error"] as? String {
            self.listener.WluperError(error: error)
            self.wluperLogManager!.post_event(event_name:WluperLogKey.NLU_ERROR_KEY.rawValue,event_value:error, session_id:self.session_id)
        }else{
            self.listener.WluperTree(tree: tree)
            let json_tree = try! JSONSerialization.data(withJSONObject: tree, options: .prettyPrinted)
            if let stree = String(data: json_tree, encoding: .utf8) {
                self.wluperLogManager!.post_event(event_name:WluperLogKey.NLU_KEY.rawValue,event_value:stree, session_id:self.session_id)
            }
        }
    }
    
    func cancelListening() {
        self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_KEY.rawValue,event_value:WluperLogKey.ASR_CANCEL_LISTENING.rawValue, session_id:self.session_id)
        self.assistant.reset()
        self.transcription=""
    }
    
    @objc func stopListening() {
        self.assistant.reset()
        self.listener.WluperLiveText(text: self.transcription)
        self.listener.WluperEndOfSpeech()
        self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_KEY.rawValue,event_value:WluperLogKey.ASR_END_OF_SPEECH.rawValue, session_id:self.session_id)
    }
    
    
    private func canAccess(withHandler handler: @escaping (Bool) -> Void) {
        SFSpeechRecognizer.requestAuthorization { status in
            if status == .authorized {
                AVAudioSession.sharedInstance().requestRecordPermission { authorized in
                    handler(authorized)
                }
            } else {
                handler(false)
            }
        }
    }
}

==> 1.1.0.tar.gz: ios/Classes/sdk/WluperSDK.swift <==
//
//  WluperSDK.swift
//  test_wluper_sdk
//
//  Created by Alfonso Farruggia on 17/09/2021.
//

import Foundation
import AVFoundation
import Speech
import SwiftUI


@available(iOS 10, *)
class WluperSDK {

    var listener:WluperInterface;
    var x_api_key:String;
    var log_api_key:String;
    var client_id:String;
    var transcription:String=""
    var session_id:String=""
    var tu_api:TUApi
    var log_api:LOGApi
    var wluperLogManager: WluperLogManager?
    private let assistant = SpeechAssist()
    private var timer:Timer?

    init(_listener : WluperInterface, _x_api_key:String, _log_api_key:String, _client_id:String){
        listener=_listener
        x_api_key=_x_api_key
        log_api_key = _log_api_key
        client_id = _client_id
        tu_api = TUApi(clientid: _client_id, xapikey: _x_api_key)
        log_api = LOGApi(clientid: _client_id, logapikey: _log_api_key)
    }

    func setWluperLogBase(_wluperLog: WluperLog){
        wluperLogManager = WluperLogManager(wluperLog:_wluperLog, logApi:log_api);
    }

    @available(iOS 10, *)
    private class SpeechAssist :NSObject , SFSpeechRecognitionTaskDelegate {
        var audioEngine: AVAudioEngine?
        var recognitionRequest: SFSpeechAudioBufferRecognitionRequest?
        var recognitionTask: SFSpeechRecognitionTask?
        let speechRecognizer = SFSpeechRecognizer(locale: Locale(identifier: "en-US"))

        deinit {
            reset()
        }

        func reset() {
            recognitionTask?.cancel()
            audioEngine?.stop()
            audioEngine = nil
            recognitionRequest = nil
            recognitionTask = nil
        }
    }

    func startListening() {
        self.session_id = UUID().uuidString
        var _error = false;

        canAccess { [self] authorized in
            guard authorized else {
                self.listener.WluperError(error: "Access denied")
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Access denied", session_id:self.session_id)
                _error = true;
                return
            }

            self.assistant.audioEngine = AVAudioEngine()

            guard let audioEngine = self.assistant.audioEngine else {
                self.listener.WluperError(error: "Unable to create audio engine")
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Unable to create audio engine", session_id:self.session_id)
                _error = true
                fatalError("Unable to create audio engine")
            }
            self.assistant.recognitionRequest = SFSpeechAudioBufferRecognitionRequest()
            guard let recognitionRequest = self.assistant.recognitionRequest else {
                self.listener.WluperError(error: "Unable to create request")
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Unable to create request", session_id:self.session_id)
                _error = true
                fatalError("Unable to create request")
            }
            recognitionRequest.shouldReportPartialResults = true
            if #available(iOS 13, *) {
                recognitionRequest.requiresOnDeviceRecognition = true
            }

            do {
                let audioSession = AVAudioSession.sharedInstance()
                try audioSession.setCategory(.record, mode: .measurement, options: .duckOthers)
                try audioSession.setActive(true, options: .notifyOthersOnDeactivation)
                let inputNode = audioEngine.inputNode

                let recordingFormat = inputNode.outputFormat(forBus: 0)
                inputNode.installTap(onBus: 0, bufferSize: 1024, format: recordingFormat) { (buffer: AVAudioPCMBuffer, when: AVAudioTime) in
                    recognitionRequest.append(buffer)
                }

                audioEngine.prepare()
                try audioEngine.start()

                assistant.recognitionTask = assistant.speechRecognizer?.recognitionTask(with: recognitionRequest ) { (result, error) in
                    if let result = result {
                        self.transcription = result.bestTranscription.formattedString
                        self.listener.WluperLiveText(text: self.transcription)
                        if (self.timer != nil){
                            self.timer?.invalidate();
                        }
                        self.timer = Timer.scheduledTimer(timeInterval: 2.0, target: self, selector: #selector(self.stopListening), userInfo: nil, repeats: false)
                    }

                }
            } catch {
                self.listener.WluperError(error: "Error transcibing audio: " + error.localizedDescription)
                self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_ERROR_KEY.rawValue,event_value:"Error transcibing audio: " + error.localizedDescription, session_id:self.session_id)
                _error = true
                assistant.reset()
            }
        }
        if (_error==false){
            self.listener.WluperASR(plain_text:"SpeechRecognizer: onReadyForSpeech");
            self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_KEY.rawValue,event_value:WluperLogKey.ASR_START_LISTENING.rawValue, session_id:self.session_id)

        }

    }

    func wluperTU(text:String){
        let tree:[String:Any] = self.tu_api.understand(text: text)

        if let error = tree["error"] as? String {
            self.listener.WluperError(error: error)
            self.wluperLogManager!.post_event(event_name:WluperLogKey.NLU_ERROR_KEY.rawValue,event_value:error, session_id:self.session_id)
        }else{
            self.listener.WluperTree(tree: tree)
            let json_tree = try! JSONSerialization.data(withJSONObject: tree, options: .prettyPrinted)
            if let stree = String(data: json_tree, encoding: .utf8) {
                self.wluperLogManager!.post_event(event_name:WluperLogKey.NLU_KEY.rawValue,event_value:stree, session_id:self.session_id)
            }
        }
    }

    func cancelListening() {
        self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_KEY.rawValue,event_value:WluperLogKey.ASR_CANCEL_LISTENING.rawValue, session_id:self.session_id)
        self.assistant.reset()
        self.transcription=""
    }

    @objc func stopListening() {
        self.assistant.reset()
        self.listener.WluperLiveText(text: self.transcription)
        self.listener.WluperEndOfSpeech()
        self.wluperLogManager!.post_event(event_name:WluperLogKey.ASR_KEY.rawValue,event_value:WluperLogKey.ASR_END_OF_SPEECH.rawValue, session_id:self.session_id)
    }


    private func canAccess(withHandler handler: @escaping (Bool) -> Void) {
        SFSpeechRecognizer.requestAuthorization { status in
            if status == .authorized {
                AVAudioSession.sharedInstance().requestRecordPermission { authorized in
                    handler(authorized)
                }
            } else {
                handler(false)
            }
        }
    }
}
